"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Types_1 = require("frontblock-generic/Types");
const Logger = require("log4js");
Logger.configure({
    appenders: {
        "frontblock-api-client": { type: 'stdout' },
    },
    categories: {
        default: { appenders: ['frontblock-api-client'], level: 'debug' }
    }
});
const logger = Logger.getLogger("frontblock-api-client");
/**
 * Frontblock api connection lib
 *
 * Can be used solo and also is a valid frontblock plugin on the customer-side
 */
class FrontblockApiClient {
    constructor(conf) {
        this.name = 'ApiClient';
        this.apikey = "";
        if (conf == null) {
            conf = {
                apiHost: "localhost",
                apiKey: "",
                apiPort: 10000,
                tls: false
            };
            this.conf = conf;
        }
        else {
            if (conf.apiKey != null)
                this.apikey = conf.apiKey;
            this.conf = conf;
        }
    }
    async consume(apikey, uid, callback) {
        const r = await this.socket.call("consume", apikey, uid);
        const res = Types_1.parseSubResponse(r);
        if (res instanceof Types_1.SubscriptionResponse) {
            this.socket.hook(res.uid, (tx) => {
                const deserialized = JSON.parse(tx);
                callback(deserialized);
            });
        }
        return res;
    }
    async quit(apikey, consumerUid) {
        const r = await this.socket.call("quit", apikey, consumerUid);
        const res = Types_1.parseResponse(r);
        return res;
    }
    async subscribe(apikey, coin, account) {
        const r = await this.socket.call("subscribe", apikey, coin, account);
        const res = Types_1.parseSubResponse(r);
        return res;
    }
    async subsume(apikey, coin, account, callback) {
        const r = await this.socket.call("subsume", apikey, coin, account);
        const res = Types_1.parseSubResponse(r);
        if (res instanceof Types_1.SubscriptionResponse) {
            this.socket.hook(res.uid, (tx) => {
                const deserialized = JSON.parse(tx);
                callback(deserialized);
            });
        }
        return res;
    }
    async unsubscribe(apikey, uid) {
        const r = await this.socket.call('unsubscribe', apikey, uid);
        const res = Types_1.parseResponse(r);
        if (res instanceof Types_1.SuccessResponse)
            this.socket.unhook(uid);
        return res;
    }
    async getPluginList() {
        return await this.socket.call('getPluginList');
    }
}
exports.default = FrontblockApiClient;
