"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const FrontblockApiClient_1 = require("./FrontblockApiClient");
const Logger = require("log4js");
Logger.configure({
    appenders: {
        "frontblock-api-client": { type: 'stdout' },
    },
    categories: {
        default: { appenders: ['frontblock-api-client'], level: 'debug' }
    }
});
const logger = Logger.getLogger("frontblock-api-client");
const bsock = require('bsock');
class FrontblockApiClientPlugin extends FrontblockApiClient_1.default {
    constructor() {
        super(...arguments);
        this.started = false;
    }
    exportRPCs() {
        const quit = async (uid) => { return await this.quit(this.apikey, uid); };
        return [
            {
                name: "subscribe",
                rpc: async (coin, account) => { return await this.subscribe(this.apikey, coin, account); },
                type: 'call',
                visibility: 'private'
            }, {
                name: "subsume",
                rpc: async (coin, account, callback) => { return await this.subsume(this.apikey, coin, account, callback); },
                type: 'hook',
                unhook: quit,
                visibility: 'private'
            }, {
                name: 'unsubscribe',
                rpc: async (uid) => { return await this.unsubscribe(this.apikey, uid); },
                type: 'unhook',
                visibility: 'private'
            }, {
                name: 'consume',
                rpc: async (uid, callback) => { return await this.consume(this.apikey, uid, callback); },
                type: 'hook',
                unhook: quit,
                visibility: 'private'
            }, {
                name: 'quit',
                rpc: async (consumerUid) => { return await this.quit(this.apikey, consumerUid); },
                type: 'call',
                visibility: 'private'
            }
        ];
    }
    start() {
        if (this.started) {
            logger.warn("FrontblockApiClient has already been started. Ignoring");
            return;
        }
        logger.info("Starting apiClient with ", this.conf);
        this.socket = bsock.connect(this.conf.apiPort, this.conf.apiHost, this.conf.tls != null ? this.conf.tls : false);
        this.started = true;
    }
    stop() {
        if (!this.started) {
            logger.warn("FrontblockApiClient has not been started. Ignoring");
            return;
        }
        this.socket.close();
    }
}
exports.default = FrontblockApiClientPlugin;
